## This script (which should turned into a vignette)
## illustrates the usage of aRT with emphasis on:
## \item reading and tidying up text files with polygon information
## \item storing on the data-base with respective attributes
## \item performing some operations on polygons such as:
##       centroids, area, perimeter and neighbourhood
## \item reading and storing point data from a case-control study
## \item performing some operation between points and polygons

## data from Silvia's thesis and papers on Biometrics and Biometrika
## proper references and aknowledgements to be added

require(maptools)
require(sp)
require(aRT)

## Reading R dump files with the polygons for each city
## and making a list with the polygons coordinates
files <- dir(path="Polygons", pattern="dump.*.poly.unit")
objs <- substring(files, 6)
cities <- unlist(strsplit(objs, ".poly.unit"))
nw <- list()
for(i in 1:length(files)){
  source(paste("Polygons/",files[i], sep=""))
  nw[[cities[i]]] <- get(objs[i])
}

## computing limits and plotting the polygons using basic \R\ functions
xl <- diag(apply(sapply(nw, function(x) range(x[,1], na.rm=T)),1,range))
yl <- diag(apply(sapply(nw, function(x) range(x[,2], na.rm=T)),1,range))
plot(xl,yl,type="n", asp=1)
sapply(nw,polygon)

## removing object no longer needed
rm(list=objs)
rm(i,objs)

## converting nw to \pkg{maptools}'s \code{polylist} class
## (is there a easier way to do so???)
attr(nw, "maplim") <- list(x=xl, y=yl)
attr(nw, "region.id") <- sprintf("%02d",1:length(nw))
attr(nw, "plotOrder") <- 1:length(nw)
attr(nw, "after") <- rep(NA, length(nw))
class(nw) <- "polylist"
for(i in 1:length(nw))
  attr(nw[[i]], "bbox") <- as.vector(t(apply(nw[[i]],2,range, na.rm=TRUE)))

## and now it can be more easily plotted by plot.polylist() from maptools
plot(nw)

## backing up
##save.image(file="nw.RData")
## restoring from backup
##load("nw.RData")

## Highlighting polygons with "problems": preston, lancaster e wyre
polygon(nw$preston, col=2)
polygon(nw$lancaster, col=3)
polygon(nw$wyre, col=4)

## looking more closely the polygons with problems
plot(subset(nw, names(nw) == "lancaster"))
plot(subset(nw, names(nw) == "preston"))
plot(subset(nw, names(nw) == "wyre"))

## checking for each polygon whether the 1st point equals to the last
## i.e. if the polygons are closed
which(sapply(nw, function(x){!identical(x[1,], x[nrow(x),])}))
#lapply(nw, function(x)
#       if(!identical(x[1,], x[nrow(x),])) x <- rbind(x,x[1,]))

## converting to the sp format:
## this does not work because of the polygons with problems:
#nwSP <- lapply(nw,Polygon)

## but notice that excluding the "problems" it does work:
nwsub <- subset(nw, !(names(nw) %in% c("preston", "lancaster", "wyre")))
nwsubSP <- lapply(nwsub, Polygon)
nwsubSP <- lapply(1:length(nwsubSP), function(x)
                  Polygons(list(nwsubSP[[x]]), ID=names(nwsub)[x]))
nwsubSP <- SpatialPolygons(nwsubSP)
plot(nwsubSP)

## This is from Pedro, and it is more general since it
## handles the problems and also
## converts the list of polygons to the sp format
resp <- lapply(1:length(nw), function(pos)
               {
                 it=nw[[pos]]
                 nas = which(is.na(it[,2])) # hole's positions
                 if(length(nas) == 0) # if no hole all is ok
                   pol = list(Polygon(it))
                 if(length(nas) > 0){
                   nas = c(0,nas,length(it[,1])+1)
                   ## adds to the holes vector de buracos 0 and hole size
                   pol = lapply(1:(length(nas)-1), function(i)
                     {Polygon(it[(nas[i]+1):(nas[i+1]-1),])
                       ## polygons with consecutive positions 
                       ## left+1 and dir=1 to exclud NA's
                       })
                 }
                 Polygons(pol,ID=names(nw)[pos])
               })

## converting "corrected" polygons to a \pkg{sp} class
nwSP <- SpatialPolygons(resp)
plot(nwSP)

## the following removes duplicated values in the polygons coordinates
## (otherwise the union of polygons using \pkg{aRT} won't work
ag=as.aRTgeometry(nwSP) -> g
removeDup(g)

## simplfying the polygon  == to be implemented!!!
##simplify(g,0.002,0.002)

## creatring data-base (requires permissions for the user on the DBMS)
con <- openConn()
#check mysql
{if(any(showDbs(con) == "northwest"))
    deleteDb(con, "northwest",force=TRUE)}
#check mysql
db <- createDb(con,"northwest")
#check mysql

## creating a layer to store the district's polygons
## and adding the geometry to the database
ldistricts <- createLayer(db,"districts")
addPolygons(ldistricts,g)

## check TV

## attachs a table to the layer (PEDRO: THIS CAN BE AUTOMATICALLY CALLED
## BY addPolygons() SINCE IS NEEDED EVEN IF THERE ARE NO ATTRIBUTES)
tpol <- createTable(ldistricts,"districts")

## reading a table of attributes associated to each polygon
leuc <- read.table("attrpolygons.dat", head=T, row.names=1)
head(leuc)

## notice the column names contains "."
## "." is a mysql keyword. replacing it to "_" ...
names(leuc) <- sub(".","_",names(leuc), fixed=TRUE)
names(leuc)

head(leuc)
leuc <- cbind(id=getID(nwSP),leuc)
head(leuc)
updateColumns(tpol,leuc)

## and writing atttributes in the database
## at this points everything is already visible from a TL applications
## such as TerraView

## check TV

# And the visualisation can be set from here
createTheme(ldistricts,view="view")

## check TV

## reversely, data can be recovered from the DBMS
getData(tpol)


##
## some aRT/TerralLib computations
##
## geometric centroid
centroids <- getOperation(ldistricts,"centroid")
centroids
centroids@coords
points(centroids@coords, pch=20)

## PJ: maybe the folowwing could returned a named vector instead of a list?
## area of each polygon
getMetric(ldistricts,"area")

## PJ: maybe the folowwing could returned a named vector instead of a list?
## perimeter of each polygon1
getMetric(ldistricts,"length")

## union of all polygons -- A BIT TIME DEMANDING!
union <- getSetOperation(ldistricts,"union",getID(nwSP))

aRTplot(union,col="gray")

## neighbourhood matrix -- VERY TIME DEMANDING!
neighs<-sapply(names(nw),
               function(x){
                 print(paste("neighbours of ",x))
                 which(is.element(names(nw),
                                  getRelation(ldistricts,"touches",id=x)))}
               )
neighs
save.image()

## map with some districs and their neighboors
ids=c("lancaster","wigan","burnley")

## currently a polygon is its on neighbour
## Pedro: I believe this must be an argumetn with defaults to false
## i.e. by default do not include the polygon in its neighbourhood
aRTplot(nwSP,add=T,id=ids,col="blue")
for(i in ids){
  aRTplot(nwSP,add=T,col="yellow",id=getID(nwSP)[neighs[[i]]])
}

## next step is to get more general neig. operations as, for instance,
## the "size" of the border between 2 polygons


##
## Points data: case-control data
## 

## now creating another layer with points (cases and controls): 

## (a) cases
cases <- read.table("cases.txt", head=T)
head(cases)
p <- SpatialPoints(cases[c("x","y")])
spdf <- SpatialPointsDataFrame(p,data.frame(ID=paste(1:nrow(p@coords))))
## Pedro: aRT should be able to transfer attributed from a
##        SpatialPointsDataFrame/creat ID automatically or use rownames

## adding attributes of the "cases" points
lcases = createLayer(db,"cases")
addPoints(lcases, spdf)
tcas=createTable(lcases,"cases")

updateColumns(tcas,cbind(id=paste(1:nrow(p@coords)),cases))
head(getData(tcas))
createTheme(lcases,view="view")

## (b) controls
controls <- read.table("controls.txt", head=T)
head(controls)
p <- SpatialPoints(controls[c("x","y")])
spdf=SpatialPointsDataFrame(p,data.frame(ID=paste(1:(length(p@coords)/2))))

## adding attributes to the "control" points
lcontrols = createLayer(db,"controls")
addPoints(lcontrols, spdf)
tcon=createTable(lcontrols,"controls")
updateColumns(tcon,cbind(id=paste(1:nrow(p@coords)),controls))

createTheme(lcontrols,view="view")


## counting how many points (\# of cases and \# of controls)
## withing each polygon
countcases=unlist(lapply(getID(nwSP), function(x){
    length(getRelation(lcases,"within",ldistricts,id=x))}))
countcases

countcontrol=unlist(lapply(getID(nwSP), function(x){
    length(getRelation(lcontrols,"within",ldistricts,id=x))}))
countcontrol


## adding these to the polygons attribute table
ncc <- data.frame(id=getID(nwSP),countcases,countcontrol)
updateColumns(tpol,ncc)
